==> ui-18.0.1.tgz: package/components/drawer/Drawer.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useId, useLockScroll, useRefExtra } from '@react-devui/hooks';
import { getClassName, toPx } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate, handleModalKeyDown, TTANSITION_DURING_BASE } from '../../utils';
import { DMask } from '../_mask';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DDrawerFooter } from './DrawerFooter';
import { DDrawerHeader } from './DrawerHeader';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DDrawer' });
export const DDrawer = (props) => {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dContainer, dPlacement = 'right', dWidth = 400, dHeight = 280, dZIndex, dMask = true, dMaskClosable = true, dEscClosable = true, dSkipFirstTransition = true, dDestroyAfterClose = true, dHeader, dFooter, dChildDrawer, onClose, afterVisibleChange, __zIndex, __onVisibleChange } = _a, restProps = __rest(_a, ["children", "dVisible", "dContainer", "dPlacement", "dWidth", "dHeight", "dZIndex", "dMask", "dMaskClosable", "dEscClosable", "dSkipFirstTransition", "dDestroyAfterClose", "dHeader", "dFooter", "dChildDrawer", "onClose", "afterVisibleChange", "__zIndex", "__onVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const drawerRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}drawer-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}drawer-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = drawerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const dataRef = useRef({
        prevActiveEl: null,
    });
    const uniqueId = useId();
    const titleId = `${dPrefix}drawer-title-${uniqueId}`;
    const bodyId = `${dPrefix}drawer-content-${uniqueId}`;
    const [distance, setDistance] = useState({
        visible: false,
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
    });
    const nestedStyle = {
        transition: `transform 140ms ${distance.visible ? 'ease-out' : 'ease-in'} 60ms`,
        transform: dPlacement === 'top'
            ? `translateY(${(distance[dPlacement] / 3) * 2}px)`
            : dPlacement === 'right'
                ? `translateX(${-(distance[dPlacement] / 3) * 2}px)`
                : dPlacement === 'bottom'
                    ? `translateY(${-(distance[dPlacement] / 3) * 2}px)`
                    : `translateX(${(distance[dPlacement] / 3) * 2}px)`,
    };
    const [visible, changeVisible] = useDValue(false, dVisible, onClose);
    const isFixed = isUndefined(dContainer);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (!isUndefined(dZIndex)) {
            return dZIndex;
        }
        if (isUndefined(__zIndex)) {
            if (isFixed) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return __zIndex;
        }
    })();
    useLockScroll(isFixed && visible);
    useEffect(() => {
        if (visible) {
            const height = isString(dHeight) ? toPx(dHeight, true) : dHeight;
            const width = isString(dWidth) ? toPx(dWidth, true) : dWidth;
            __onVisibleChange === null || __onVisibleChange === void 0 ? void 0 : __onVisibleChange({
                visible: true,
                top: distance.top + (dPlacement === 'top' ? height : 0),
                right: distance.right + (dPlacement === 'right' ? width : 0),
                bottom: distance.bottom + (dPlacement === 'bottom' ? height : 0),
                left: distance.left + (dPlacement === 'left' ? width : 0),
            });
        }
        else {
            __onVisibleChange === null || __onVisibleChange === void 0 ? void 0 : __onVisibleChange(Object.assign(Object.assign({}, distance), { visible: false }));
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [visible]);
    useEffect(() => {
        if (visible) {
            dataRef.current.prevActiveEl = document.activeElement;
            if (drawerRef.current) {
                drawerRef.current.focus({ preventScroll: true });
            }
        }
        else if (dataRef.current.prevActiveEl) {
            dataRef.current.prevActiveEl.focus({ preventScroll: true });
        }
    }, [visible]);
    const transitionStyles = (() => {
        const transform = dPlacement === 'top'
            ? 'translate(0, -100%)'
            : dPlacement === 'right'
                ? 'translate(100%, 0)'
                : dPlacement === 'bottom'
                    ? 'translate(0, 100%)'
                    : 'translate(-100%, 0)';
        return {
            enter: { transform },
            entering: {
                transition: ['transform'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-out`).join(', '),
            },
            leaving: {
                transform,
                transition: ['transform'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-in`).join(', '),
            },
            leaved: { transform },
        };
    })();
    const childDrawer = (() => {
        if (dChildDrawer) {
            return React.cloneElement(dChildDrawer, {
                __zIndex: isUndefined(zIndex) ? zIndex : `calc(${zIndex} + 1)`,
                __onVisibleChange: (distance) => {
                    setDistance(distance);
                },
            });
        }
        return null;
    })();
    const headerNode = (() => {
        if (dHeader) {
            const node = isString(dHeader) ? _jsx(DDrawerHeader, { children: dHeader }) : dHeader;
            return React.cloneElement(node, {
                __id: titleId,
                __onClose: () => {
                    changeVisible(false);
                },
            });
        }
    })();
    const drawerNode = (_jsxs(_Fragment, { children: [_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING_BASE, dSkipFirstTransition: dSkipFirstTransition, dDestroyWhenLeaved: dDestroyAfterClose, afterEnter: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                }, afterLeave: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                } }, { children: (state) => {
                    var _a;
                    return (_jsxs("div", Object.assign({}, restProps, { ref: drawerRef, className: getClassName(restProps.className, `${dPrefix}drawer`), style: Object.assign(Object.assign(Object.assign({}, restProps.style), nestedStyle), { display: state === 'leaved' ? 'none' : undefined, position: isFixed ? undefined : 'absolute', zIndex }), tabIndex: (_a = restProps.tabIndex) !== null && _a !== void 0 ? _a : -1, role: "dialog", "aria-modal": true, "aria-labelledby": headerNode ? titleId : undefined, "aria-describedby": bodyId, onKeyDown: (e) => {
                            var _a;
                            (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                            if (dEscClosable && e.code === 'Escape') {
                                changeVisible(false);
                            }
                            handleModalKeyDown(e);
                        } }, { children: [dMask && (_jsx(DMask, { dVisible: visible, onClose: () => {
                                    if (dMaskClosable) {
                                        changeVisible(false);
                                    }
                                } })), _jsxs("div", Object.assign({ className: getClassName(`${dPrefix}drawer__content`, `${dPrefix}drawer__content--${dPlacement}`), style: Object.assign({ width: dPlacement === 'left' || dPlacement === 'right' ? dWidth : undefined, height: dPlacement === 'bottom' || dPlacement === 'top' ? dHeight : undefined }, transitionStyles[state]) }, { children: [headerNode, _jsx("div", Object.assign({ id: bodyId, className: `${dPrefix}drawer__body` }, { children: children })), dFooter &&
                                        React.cloneElement(dFooter, {
                                            __onClose: () => {
                                                changeVisible(false);
                                            },
                                        })] }))] })));
                } })), childDrawer] }));
    return dContainer === false ? drawerNode : containerRef.current && ReactDOM.createPortal(drawerNode, containerRef.current);
};
DDrawer.Header = DDrawerHeader;
DDrawer.Footer = DDrawerFooter;

==> ui-18.1.3.tgz: package/components/drawer/Drawer.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useId, useLockScroll, useRefExtra } from '@react-devui/hooks';
import { getClassName, toPx } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate, handleModalKeyDown, TTANSITION_DURING_BASE, checkNoExpandedEl } from '../../utils';
import { DMask } from '../_mask';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DDrawerFooter } from './DrawerFooter';
import { DDrawerHeader } from './DrawerHeader';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DDrawer' });
export const DDrawer = (props) => {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dContainer, dPlacement = 'right', dWidth = 400, dHeight = 280, dZIndex, dMask = true, dMaskClosable = true, dEscClosable = true, dSkipFirstTransition = true, dDestroyAfterClose = true, dHeader, dFooter, dChildDrawer, onClose, afterVisibleChange, __zIndex, __onVisibleChange } = _a, restProps = __rest(_a, ["children", "dVisible", "dContainer", "dPlacement", "dWidth", "dHeight", "dZIndex", "dMask", "dMaskClosable", "dEscClosable", "dSkipFirstTransition", "dDestroyAfterClose", "dHeader", "dFooter", "dChildDrawer", "onClose", "afterVisibleChange", "__zIndex", "__onVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const drawerRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}drawer-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}drawer-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = drawerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const dataRef = useRef({
        prevActiveEl: null,
    });
    const uniqueId = useId();
    const titleId = `${dPrefix}drawer-title-${uniqueId}`;
    const bodyId = `${dPrefix}drawer-content-${uniqueId}`;
    const [distance, setDistance] = useState({
        visible: false,
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
    });
    const nestedStyle = {
        transition: `transform 140ms ${distance.visible ? 'ease-out' : 'ease-in'} 60ms`,
        transform: dPlacement === 'top'
            ? `translateY(${(distance[dPlacement] / 3) * 2}px)`
            : dPlacement === 'right'
                ? `translateX(${-(distance[dPlacement] / 3) * 2}px)`
                : dPlacement === 'bottom'
                    ? `translateY(${-(distance[dPlacement] / 3) * 2}px)`
                    : `translateX(${(distance[dPlacement] / 3) * 2}px)`,
    };
    const [visible, changeVisible] = useDValue(false, dVisible, onClose);
    const isFixed = isUndefined(dContainer);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (!isUndefined(dZIndex)) {
            return dZIndex;
        }
        if (isUndefined(__zIndex)) {
            if (isFixed) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return __zIndex;
        }
    })();
    useLockScroll(isFixed && visible);
    useEffect(() => {
        if (visible) {
            const height = isString(dHeight) ? toPx(dHeight, true) : dHeight;
            const width = isString(dWidth) ? toPx(dWidth, true) : dWidth;
            __onVisibleChange === null || __onVisibleChange === void 0 ? void 0 : __onVisibleChange({
                visible: true,
                top: distance.top + (dPlacement === 'top' ? height : 0),
                right: distance.right + (dPlacement === 'right' ? width : 0),
                bottom: distance.bottom + (dPlacement === 'bottom' ? height : 0),
                left: distance.left + (dPlacement === 'left' ? width : 0),
            });
        }
        else {
            __onVisibleChange === null || __onVisibleChange === void 0 ? void 0 : __onVisibleChange(Object.assign(Object.assign({}, distance), { visible: false }));
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [visible]);
    useEffect(() => {
        if (visible) {
            dataRef.current.prevActiveEl = document.activeElement;
            if (drawerRef.current) {
                drawerRef.current.focus({ preventScroll: true });
            }
        }
        else if (dataRef.current.prevActiveEl) {
            dataRef.current.prevActiveEl.focus({ preventScroll: true });
        }
    }, [visible]);
    const transitionStyles = (() => {
        const transform = dPlacement === 'top'
            ? 'translate(0, -100%)'
            : dPlacement === 'right'
                ? 'translate(100%, 0)'
                : dPlacement === 'bottom'
                    ? 'translate(0, 100%)'
                    : 'translate(-100%, 0)';
        return {
            enter: { transform },
            entering: {
                transition: ['transform'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-out`).join(', '),
            },
            leaving: {
                transform,
                transition: ['transform'].map((attr) => `${attr} ${TTANSITION_DURING_BASE}ms ease-in`).join(', '),
            },
            leaved: { transform },
        };
    })();
    const childDrawer = (() => {
        if (dChildDrawer) {
            return React.cloneElement(dChildDrawer, {
                __zIndex: isUndefined(zIndex) ? zIndex : `calc(${zIndex} + 1)`,
                __onVisibleChange: (distance) => {
                    setDistance(distance);
                },
            });
        }
        return null;
    })();
    const headerNode = (() => {
        if (dHeader) {
            const node = isString(dHeader) ? _jsx(DDrawerHeader, { children: dHeader }) : dHeader;
            return React.cloneElement(node, {
                __id: titleId,
                __onClose: () => {
                    changeVisible(false);
                },
            });
        }
    })();
    const drawerNode = (_jsxs(_Fragment, { children: [_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING_BASE, dSkipFirstTransition: dSkipFirstTransition, dDestroyWhenLeaved: dDestroyAfterClose, afterEnter: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                }, afterLeave: () => {
                    afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                } }, { children: (state) => {
                    var _a;
                    return (_jsxs("div", Object.assign({}, restProps, { ref: drawerRef, className: getClassName(restProps.className, `${dPrefix}drawer`), style: Object.assign(Object.assign(Object.assign({}, restProps.style), nestedStyle), { display: state === 'leaved' ? 'none' : undefined, position: isFixed ? undefined : 'absolute', zIndex }), tabIndex: (_a = restProps.tabIndex) !== null && _a !== void 0 ? _a : -1, role: "dialog", "aria-modal": true, "aria-labelledby": headerNode ? titleId : undefined, "aria-describedby": bodyId, onKeyDown: (e) => {
                            var _a;
                            (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                            if (dEscClosable && checkNoExpandedEl(e.currentTarget) && e.code === 'Escape') {
                                changeVisible(false);
                            }
                            handleModalKeyDown(e);
                        } }, { children: [dMask && (_jsx(DMask, { dVisible: visible, onClose: () => {
                                    if (dMaskClosable) {
                                        changeVisible(false);
                                    }
                                } })), _jsxs("div", Object.assign({ className: getClassName(`${dPrefix}drawer__content`, `${dPrefix}drawer__content--${dPlacement}`), style: Object.assign({ width: dPlacement === 'left' || dPlacement === 'right' ? dWidth : undefined, height: dPlacement === 'bottom' || dPlacement === 'top' ? dHeight : undefined }, transitionStyles[state]) }, { children: [headerNode, _jsx("div", Object.assign({ id: bodyId, className: `${dPrefix}drawer__body` }, { children: children })), dFooter &&
                                        React.cloneElement(dFooter, {
                                            __onClose: () => {
                                                changeVisible(false);
                                            },
                                        })] }))] })));
                } })), childDrawer] }));
    return dContainer === false ? drawerNode : containerRef.current && ReactDOM.createPortal(drawerNode, containerRef.current);
};
DDrawer.Header = DDrawerHeader;
DDrawer.Footer = DDrawerFooter;
